<?php
/**
 * Plugin Name: Go Personal Plugin
 * Plugin URI: https://gopersonal.com
 * Description: Custom plugin to inject Go Personal SDK
 * Version: 1.6.0
 * Author: Go Personal
 * Author URI: https://gopersonal.com
 */

// Hook to add admin menu
add_action('admin_menu', 'gopersonal_plugin_menu');

function gopersonal_plugin_menu() {
    // Add an item to the menu for the settings page
    add_menu_page('Go Personal Plugin Settings', 'Go Personal', 'manage_options', 'go-personal-plugin', 'gopersonal_plugin_settings_page');
}

// Create the settings page content
function gopersonal_plugin_settings_page() {
    ?>
    <div class="wrap">
        <h1>Go Personal Plugin Settings</h1>
        <form method="post" action="options.php">
            <?php
            settings_fields('gopersonal-plugin-settings');
            do_settings_sections('go-personal-plugin');
            submit_button();
            ?>
        </form>
    </div>
    <?php
}

add_action('admin_init', 'gopersonal_plugin_settings_init');

function gopersonal_plugin_settings_init() {
    // Register a new setting for "clientId"
    register_setting('gopersonal-plugin-settings', 'gopersonal_plugin_clientId');

    // Add a new section in our admin page
    add_settings_section(
        'gopersonal_plugin_settings_section',
        'Go Personal Plugin Settings',
        'gopersonal_plugin_settings_section_cb',
        'go-personal-plugin'
    );

    // Add the field for clientId
    add_settings_field(
        'gopersonal_plugin_clientId',
        'Client ID',
        'gopersonal_plugin_settings_field_clientId_cb',
        'go-personal-plugin',
        'gopersonal_plugin_settings_section'
    );
}

// Section callback function
function gopersonal_plugin_settings_section_cb() {
    echo '<p>Enter your settings below:</p>';
}

// ClientId field callback function
function gopersonal_plugin_settings_field_clientId_cb() {
    // Get the clientId value from the database
    $clientId = get_option('gopersonal_plugin_clientId');
    // Output the field
    echo '<input type="text" id="gopersonal_plugin_clientId" name="gopersonal_plugin_clientId" value="' . esc_attr($clientId) . '"/>';
}

// Hook into wp_head to print JavaScript in the head section
add_action('wp_head', 'gopersonal_print_js');

function gopersonal_print_js() {
    $clientId = get_option('gopersonal_plugin_clientId', 'default_clientId');

    // Determine the page type
    $pageType = 'other'; // default page type to 'other'

    if (is_front_page() || is_home()) {
        // Set page type to 'home' if on the homepage
        $pageType = 'home';
    } elseif (is_cart()) {
        // Set page type to 'cart' if on the cart page
        $pageType = 'cart';
    } elseif (is_checkout()) {
        // Set page type to 'checkout' if on the checkout page
        $pageType = 'checkout';
    } elseif (is_product()) {
        // Set page type to 'product_detail' if on a product detail page
        $pageType = 'product_detail';
    }

    // Get the product ID if on a product detail page
    $productId = '';
    if ($pageType === 'product_detail') {
        $productId = get_the_ID();
    }

    // Get customer ID if user is logged in
    $customerId = '';
    if (is_user_logged_in()) {
        $current_user = wp_get_current_user();
        $customerId = $current_user->ID;
    }

    // JavaScript code as a string
    $js_code = "<script>
    function setGoPersonalJWT(jwt, attempt = 1) {
        fetch('/wp-admin/admin-ajax.php', {
            method: 'POST',
            credentials: 'same-origin', // Needed for WordPress to recognize the logged-in user
            headers: {
                'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8'
            },
            body: 'action=set_gopersonal_jwt&jwt=' + encodeURIComponent(jwt)
        })
        .then(response => response.json())
        .then(data => {
            if (data.success) {
                console.log('Success:', data.data.message);
            } else {
                throw new Error(data.data.message);
            }
        })
        .catch((error) => {
            console.error('Retry attempt ' + attempt + ' failed:', error);
            if (attempt < 3) { // retry up to 3 times
                setTimeout(() => setGoPersonalJWT(jwt, attempt + 1), 2000); // retry after 2 seconds
            }
        });
    };
    async function initializeGSSDK() {
        var context = { 
            pageType: '" . esc_js($pageType) . "', 
            customerId: '" . (is_user_logged_in() ? esc_js(get_current_user_id()) : '') . "'";

    if ($pageType === 'product_detail') {
      $js_code .= ", product_id: '" . esc_js($productId) . "'";
    }

    $js_code .= "};
    
    window.gsSDK = await new window.GSSDK.default('" . esc_js($clientId) . "', {provider: 'WooCommerce', context: context});";
    
    if (is_user_logged_in()) {
      $current_user = wp_get_current_user();
      $js_code .= "
          window.gsSDK.login('" . esc_js($current_user->ID) . "');";
      }

    $js_code .= "
        setTimeout(function() {
            setGoPersonalJWT(window.gsSDK.getCurrentSession().token);
        }, 500);
    }

    var gsSDKScript = document.createElement('script');
    gsSDKScript.src = 'https://gs-sdk.pages.dev/gs-sdk.js';
    gsSDKScript.onload = initializeGSSDK;
    document.head.appendChild(gsSDKScript);
    </script>";

    echo $js_code;
}


add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'gopersonal_add_settings_link');

function gopersonal_add_settings_link($links) {
    // Build and add the settings link
    $settings_link = '<a href="' . admin_url('admin.php?page=go-personal-plugin') . '">Settings</a>';
    array_unshift($links, $settings_link);

    return $links;
}

add_action('wp_ajax_get_gopersonal_jwt', 'get_gopersonal_jwt_ajax');
add_action('wp_ajax_nopriv_get_gopersonal_jwt', 'get_gopersonal_jwt_ajax');

function get_gopersonal_jwt_ajax() {
    // Ensure WooCommerce session is available
    if (function_exists('WC') && WC()->session) {
        $jwt_value = get_gopersonal_jwt();
        
        if (!empty($jwt_value)) {
            wp_send_json_success(['jwt' => $jwt_value]);
        } else {
            wp_send_json_error(['message' => 'JWT value is not set']);
        }
    } else {
        wp_send_json_error(['message' => 'WooCommerce session not available']);
    }
    exit;
}

add_action('wp_ajax_fetch_cart_items', 'fetch_cart_items');
add_action('wp_ajax_nopriv_fetch_cart_items', 'fetch_cart_items');


function fetch_cart_items() {
  $user_id = get_current_user_id(); // Returns 0 if no user is logged in
  $session_token = $user_id ? $user_id : WC()->session->get_customer_id();

  // Retrieve cart items using WooCommerce functions
  $cart_items = WC()->cart->get_cart();

  // Prepare the response
  $response = array();
  foreach ($cart_items as $cart_item_key => $cart_item) {
    $product = $cart_item['data'];
    $global_unique_key = $session_token . '_' . $cart_item_key;

    // Customize the response as needed
    $response[] = array(
      'cart_item_key' => $cart_item_key, // Unique identifier for the cart item
      'global_unique_key' => $global_unique_key,
      'product_id' => $product->get_id(), // Product ID
      'product_name' => $product->get_name(),
      'quantity' => $cart_item['quantity'],
      'price' => $product->get_price(),
    );
  }

  // Send the response as JSON
  wp_send_json($response);
}

add_action('wp_ajax_fetch_user_status', 'fetch_user_status');
add_action('wp_ajax_nopriv_fetch_user_status', 'fetch_user_status');

function fetch_user_status() {
    $response = array(
        'is_logged_in' => is_user_logged_in(),
        'user_id' => get_current_user_id()
    );

    wp_send_json($response);
}

// Add the action hook
function get_gopersonal_jwt() {
    if (WC()->session) {
        $jwt_value = WC()->session->get('gopersonal_jwt');
        if (!empty($jwt_value)) {
            return $jwt_value;
        } else {
            // Handle the case where the JWT is not set or is empty
            return null;
        }
    } else {
        // Handle the case where the WooCommerce session is not available
        return null;
    }
}

function post_data_on_purchase_completed($order_id) {
  $data = array(
    'transactionId' => $order_id
  );
  send_post_discover($data, '/interaction/state/cart');
}

function action_woocommerce_add_to_cart($cart_item_key, $product_id, $quantity, $variation_id, $variation, $cart_item_data) {
    // Static variable to hold the cart item keys that have been processed
    static $processed_cart_items = array();
  
    // Check if this cart item key has already been processed
    if (in_array($cart_item_key, $processed_cart_items)) {
        // If yes, do not process it again
        return;
    }
  
    // Add the cart item key to the array to mark it as processed
    $processed_cart_items[] = $cart_item_key;

    // Your existing data handling
    $data = array(
        'event' => 'cart',
        'item' => $product_id,
        'quantity' => $quantity,
        'transactionId' => $cart_item_key
    );

    send_post_discover($data, '/interaction');
}

add_action('woocommerce_add_to_cart', 'action_woocommerce_add_to_cart', 10, 6);

function action_woocommerce_remove_cart_item($cart_item_key, $cart) {
    // Retrieve the cart item
    $cart_item = $cart->cart_contents[$cart_item_key];

    // Get the product ID
    $product_id = $cart_item['product_id'];

    // Get the quantity of the item
    $quantity = $cart_item['quantity'];

    $data = array(
        'event' => 'remove-cart',
        'item' => $product_id,
        'quantity' => $quantity
    );

    send_post_discover($data, '/interaction');
}
add_action('woocommerce_remove_cart_item', 'action_woocommerce_remove_cart_item', 10, 2);

function on_cart_quantity_update($cart_item_key, $quantity, $old_quantity, $cart) {
    // Check if the quantity has changed
    if ($quantity != $old_quantity) {
      $cart_item = $cart->cart_contents[$cart_item_key];
  
      // Safety check - Ensure the cart item exists
      if (!$cart_item) {
          return;
      }
  
      // Get the product ID
      $product_id = $cart_item['product_id'];
  
      // Determine if the quantity has increased or decreased
      if ($quantity > $old_quantity) {
          $quantity_added = $quantity - $old_quantity;
          $data = array(
              'event' => 'cart',
              'item' => $product_id,
              'quantity' => $quantity_added
          );
          send_post_discover($data, '/interaction');
      } else {
          $quantity_removed = $old_quantity - $quantity;
          $data = array(
              'event' => 'remove-cart',
              'item' => $product_id,
              'quantity' => $quantity_removed
          );
          send_post_discover($data, '/interaction');
      }
    }
  }

// Add the action hook
add_action('woocommerce_after_cart_item_quantity_update', 'on_cart_quantity_update', 10, 4);


// Add the action hook
add_action('woocommerce_thankyou', 'post_data_on_purchase_completed', 10, 1);

add_action('wp_ajax_set_gopersonal_jwt', 'set_gopersonal_jwt');
add_action('wp_ajax_nopriv_set_gopersonal_jwt', 'set_gopersonal_jwt');

function set_gopersonal_jwt() {
    // Ensure WooCommerce and its session handler are available
    if (function_exists('WC') && WC()->session) {
        // Check if the jwt value is set in the request

        if (!WC()->session->has_session()) {
            WC()->session->set_customer_session_cookie(true);
        }
        
        if (isset($_POST['jwt'])) {
            $jwt_value = sanitize_text_field($_POST['jwt']);

            // Set and save the value in the WooCommerce session
            WC()->session->set('gopersonal_jwt', $jwt_value);
            WC()->session->save_data();

            // Retrieve for verification
            $jwt_retrieved = WC()->session->get('gopersonal_jwt');

            wp_send_json_success([
                'message' => 'JWT value set successfully',
                'jwt_retrieved' => $jwt_retrieved
            ]);
        } else {
            wp_send_json_error(['message' => 'No JWT value provided']);
        }
    } else {
        wp_send_json_error(['message' => 'WooCommerce session not available']);
    }
    exit;
}


function send_post_discover($data, $path) {
    // Callback function for the actual HTTP request
    $http_request_callback = function() use ($data, $path) {
        // Base URL
        $url = 'https://discover.gopersonal.ai';

        // Get the clientId option
        $clientId = get_option('gopersonal_plugin_clientId');

        // If clientId starts with 'D-', use the development base URL
        if (strpos($clientId, 'D-') === 0) {
            $url = 'https://go-discover-dev.goshops.ai';
        }

        // Append the path to the URL
        $url .= $path;

        // Get the JWT
        $jwt = get_gopersonal_jwt();

        // Encode the data to JSON format
        $body = json_encode($data);

        // Set up the arguments for wp_remote_post
        $args = array(
            'body'        => $body,
            'headers'     => array(
                'Content-Type'  => 'application/json',
                'Authorization' => 'Bearer ' . $jwt
            ),
            'timeout'     => 60,
            'redirection' => 5,
            'blocking'    => true,
            'httpversion' => '1.0',
            'sslverify'   => true,
            'data_format' => 'body',
        );

        return wp_remote_post($url, $args);
    };

    // Use the retry function with the callback
    return retry_with_backoff($http_request_callback);
}

function retry_with_backoff($callback, $max_retries = 3, $initial_delay = 1) {
    $attempt = 0;
    $delay = $initial_delay;

    while ($attempt < $max_retries) {
        $response = call_user_func($callback);

        // Check for a successful response (modify this condition as needed)
        if (!is_wp_error($response) && wp_remote_retrieve_response_code($response) == 200) {
            return $response;
        }

        // Wait for a bit before retrying
        sleep($delay);

        // Increment the attempt counter and exponentially increase the delay
        $attempt++;
        $delay *= 2;
    }

    return new WP_Error('http_request_failed', 'HTTP request failed after retrying.');
}



